/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef XEN_SR_HEALTH_H
#define XEN_SR_HEALTH_H


#include <xen/api/xen_common.h>


enum xen_sr_health
{
    /**
     * Storage is fully available
     */
    XEN_SR_HEALTH_HEALTHY,

    /**
     * Storage is busy recovering, e.g. rebuilding mirrors.
     */
    XEN_SR_HEALTH_RECOVERING,

    /**
     * Storage is unreachable
     */
    XEN_SR_HEALTH_UNREACHABLE,

    /**
     * Storage is unavailable
     */
    XEN_SR_HEALTH_UNAVAILABLE,

    /**
     * Unknown to this SDK version.
     */
    XEN_SR_HEALTH_UNDEFINED
};


typedef struct xen_sr_health_set
{
    size_t size;
    enum xen_sr_health contents[];
} xen_sr_health_set;

/**
 * Allocate a xen_sr_health_set of the given size.
 */
extern xen_sr_health_set *
xen_sr_health_set_alloc(size_t size);

/**
 * Free the given xen_sr_health_set.  The given set must
 * have been allocated by this library.
 */
extern void
xen_sr_health_set_free(xen_sr_health_set *set);


/**
 * Return the name corresponding to the given code.  This string must
 * not be modified or freed.
 */
extern const char *
xen_sr_health_to_string(enum xen_sr_health val);


/**
 * Return the correct code for the given string, or set the session
 * object to failure and return an undefined value if the given string does
 * not match a known code.
 */
extern enum xen_sr_health
xen_sr_health_from_string(xen_session *session, const char *str);


#endif
