/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using Newtonsoft.Json;


namespace XenAPI
{
    [JsonConverter(typeof(on_softreboot_behaviorConverter))]
    public enum on_softreboot_behavior
    {
        /// <summary>
        /// perform soft-reboot
        /// </summary>
        soft_reboot,
        /// <summary>
        /// destroy the VM state
        /// </summary>
        destroy,
        /// <summary>
        /// restart the VM
        /// </summary>
        restart,
        /// <summary>
        /// leave the VM paused
        /// </summary>
        preserve,
        unknown
    }

    public static class on_softreboot_behavior_helper
    {
        public static string ToString(on_softreboot_behavior x)
        {
            return x.StringOf();
        }
    }

    public static partial class EnumExt
    {
        public static string StringOf(this on_softreboot_behavior x)
        {
            switch (x)
            {
                case on_softreboot_behavior.soft_reboot:
                    return "soft_reboot";
                case on_softreboot_behavior.destroy:
                    return "destroy";
                case on_softreboot_behavior.restart:
                    return "restart";
                case on_softreboot_behavior.preserve:
                    return "preserve";
                default:
                    return "unknown";
            }
        }
    }

    internal class on_softreboot_behaviorConverter : XenEnumConverter
    {
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            writer.WriteValue(((on_softreboot_behavior)value).StringOf());
        }
    }
}
