/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package com.xensource.xenapi;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonValue;
import com.fasterxml.jackson.core.type.TypeReference;
import com.xensource.xenapi.Types.BadServerResponse;
import com.xensource.xenapi.Types.XenAPIException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.*;
import java.io.IOException;

/**
 * A virtual network interface
 * First published in XenServer 4.0.
 *
 * @author Cloud Software Group, Inc.
 */
public class VIF extends XenAPIObject {

    /**
     * The XenAPI reference (OpaqueRef) to this object.
     */
    protected final String ref;

    /**
     * For internal use only.
     */
    VIF(String ref) {
       this.ref = ref;
    }

    /**
     * @return The XenAPI reference (OpaqueRef) to this object.
     */
    @JsonValue
    public String toWireString() {
       return this.ref;
    }

    /**
     * If obj is a VIF, compares XenAPI references for equality.
     */
    @Override
    public boolean equals(Object obj)
    {
        if (obj instanceof VIF)
        {
            VIF other = (VIF) obj;
            return other.ref.equals(this.ref);
        } else
        {
            return false;
        }
    }

    @Override
    public int hashCode()
    {
        return ref.hashCode();
    }

    /**
     * Represents all the fields in a VIF
     */
    public static class Record implements Types.Record {
        public String toString() {
            StringWriter writer = new StringWriter();
            PrintWriter print = new PrintWriter(writer);
            print.printf("%1$20s: %2$s\n", "uuid", this.uuid);
            print.printf("%1$20s: %2$s\n", "allowedOperations", this.allowedOperations);
            print.printf("%1$20s: %2$s\n", "currentOperations", this.currentOperations);
            print.printf("%1$20s: %2$s\n", "device", this.device);
            print.printf("%1$20s: %2$s\n", "network", this.network);
            print.printf("%1$20s: %2$s\n", "VM", this.VM);
            print.printf("%1$20s: %2$s\n", "MAC", this.MAC);
            print.printf("%1$20s: %2$s\n", "MTU", this.MTU);
            print.printf("%1$20s: %2$s\n", "otherConfig", this.otherConfig);
            print.printf("%1$20s: %2$s\n", "currentlyAttached", this.currentlyAttached);
            print.printf("%1$20s: %2$s\n", "statusCode", this.statusCode);
            print.printf("%1$20s: %2$s\n", "statusDetail", this.statusDetail);
            print.printf("%1$20s: %2$s\n", "runtimeProperties", this.runtimeProperties);
            print.printf("%1$20s: %2$s\n", "qosAlgorithmType", this.qosAlgorithmType);
            print.printf("%1$20s: %2$s\n", "qosAlgorithmParams", this.qosAlgorithmParams);
            print.printf("%1$20s: %2$s\n", "qosSupportedAlgorithms", this.qosSupportedAlgorithms);
            print.printf("%1$20s: %2$s\n", "metrics", this.metrics);
            print.printf("%1$20s: %2$s\n", "MACAutogenerated", this.MACAutogenerated);
            print.printf("%1$20s: %2$s\n", "lockingMode", this.lockingMode);
            print.printf("%1$20s: %2$s\n", "ipv4Allowed", this.ipv4Allowed);
            print.printf("%1$20s: %2$s\n", "ipv6Allowed", this.ipv6Allowed);
            print.printf("%1$20s: %2$s\n", "ipv4ConfigurationMode", this.ipv4ConfigurationMode);
            print.printf("%1$20s: %2$s\n", "ipv4Addresses", this.ipv4Addresses);
            print.printf("%1$20s: %2$s\n", "ipv4Gateway", this.ipv4Gateway);
            print.printf("%1$20s: %2$s\n", "ipv6ConfigurationMode", this.ipv6ConfigurationMode);
            print.printf("%1$20s: %2$s\n", "ipv6Addresses", this.ipv6Addresses);
            print.printf("%1$20s: %2$s\n", "ipv6Gateway", this.ipv6Gateway);
            return writer.toString();
        }

        /**
         * Convert a VIF.Record to a Map
         */
        public Map<String,Object> toMap() {
            var map = new HashMap<String,Object>();
            map.put("uuid", this.uuid == null ? "" : this.uuid);
            map.put("allowed_operations", this.allowedOperations == null ? new LinkedHashSet<Types.VifOperations>() : this.allowedOperations);
            map.put("current_operations", this.currentOperations == null ? new HashMap<String, Types.VifOperations>() : this.currentOperations);
            map.put("device", this.device == null ? "" : this.device);
            map.put("network", this.network == null ? new Network("OpaqueRef:NULL") : this.network);
            map.put("VM", this.VM == null ? new VM("OpaqueRef:NULL") : this.VM);
            map.put("MAC", this.MAC == null ? "" : this.MAC);
            map.put("MTU", this.MTU == null ? 0 : this.MTU);
            map.put("other_config", this.otherConfig == null ? new HashMap<String, String>() : this.otherConfig);
            map.put("currently_attached", this.currentlyAttached == null ? false : this.currentlyAttached);
            map.put("status_code", this.statusCode == null ? 0 : this.statusCode);
            map.put("status_detail", this.statusDetail == null ? "" : this.statusDetail);
            map.put("runtime_properties", this.runtimeProperties == null ? new HashMap<String, String>() : this.runtimeProperties);
            map.put("qos_algorithm_type", this.qosAlgorithmType == null ? "" : this.qosAlgorithmType);
            map.put("qos_algorithm_params", this.qosAlgorithmParams == null ? new HashMap<String, String>() : this.qosAlgorithmParams);
            map.put("qos_supported_algorithms", this.qosSupportedAlgorithms == null ? new LinkedHashSet<String>() : this.qosSupportedAlgorithms);
            map.put("metrics", this.metrics == null ? new VIFMetrics("OpaqueRef:NULL") : this.metrics);
            map.put("MAC_autogenerated", this.MACAutogenerated == null ? false : this.MACAutogenerated);
            map.put("locking_mode", this.lockingMode == null ? Types.VifLockingMode.NETWORK_DEFAULT : this.lockingMode);
            map.put("ipv4_allowed", this.ipv4Allowed == null ? new LinkedHashSet<String>() : this.ipv4Allowed);
            map.put("ipv6_allowed", this.ipv6Allowed == null ? new LinkedHashSet<String>() : this.ipv6Allowed);
            map.put("ipv4_configuration_mode", this.ipv4ConfigurationMode == null ? Types.VifIpv4ConfigurationMode.UNRECOGNIZED : this.ipv4ConfigurationMode);
            map.put("ipv4_addresses", this.ipv4Addresses == null ? new LinkedHashSet<String>() : this.ipv4Addresses);
            map.put("ipv4_gateway", this.ipv4Gateway == null ? "" : this.ipv4Gateway);
            map.put("ipv6_configuration_mode", this.ipv6ConfigurationMode == null ? Types.VifIpv6ConfigurationMode.UNRECOGNIZED : this.ipv6ConfigurationMode);
            map.put("ipv6_addresses", this.ipv6Addresses == null ? new LinkedHashSet<String>() : this.ipv6Addresses);
            map.put("ipv6_gateway", this.ipv6Gateway == null ? "" : this.ipv6Gateway);
            return map;
        }

        /**
         * Unique identifier/object reference
         */
        @JsonProperty("uuid")
        public String uuid;

        /**
         * list of the operations allowed in this state. This list is advisory only and the server state may have changed by the time this field is read by a client.
         */
        @JsonProperty("allowed_operations")
        public Set<Types.VifOperations> allowedOperations;

        /**
         * links each of the running tasks using this object (by reference) to a current_operation enum which describes the nature of the task.
         */
        @JsonProperty("current_operations")
        public Map<String, Types.VifOperations> currentOperations;

        /**
         * order in which VIF backends are created by xapi
         */
        @JsonProperty("device")
        public String device;

        /**
         * virtual network to which this vif is connected
         */
        @JsonProperty("network")
        public Network network;

        /**
         * virtual machine to which this vif is connected
         */
        @JsonProperty("VM")
        public VM VM;

        /**
         * ethernet MAC address of virtual interface, as exposed to guest
         */
        @JsonProperty("MAC")
        public String MAC;

        /**
         * MTU in octets
         */
        @JsonProperty("MTU")
        public Long MTU;

        /**
         * additional configuration
         */
        @JsonProperty("other_config")
        public Map<String, String> otherConfig;

        /**
         * is the device currently attached (erased on reboot)
         */
        @JsonProperty("currently_attached")
        public Boolean currentlyAttached;

        /**
         * error/success code associated with last attach-operation (erased on reboot)
         */
        @JsonProperty("status_code")
        public Long statusCode;

        /**
         * error/success information associated with last attach-operation status (erased on reboot)
         */
        @JsonProperty("status_detail")
        public String statusDetail;

        /**
         * Device runtime properties
         */
        @JsonProperty("runtime_properties")
        public Map<String, String> runtimeProperties;

        /**
         * QoS algorithm to use
         */
        @JsonProperty("qos_algorithm_type")
        public String qosAlgorithmType;

        /**
         * parameters for chosen QoS algorithm
         */
        @JsonProperty("qos_algorithm_params")
        public Map<String, String> qosAlgorithmParams;

        /**
         * supported QoS algorithms for this VIF
         */
        @JsonProperty("qos_supported_algorithms")
        public Set<String> qosSupportedAlgorithms;

        /**
         * metrics associated with this VIF
         */
        @JsonProperty("metrics")
        public VIFMetrics metrics;

        /**
         * true if the MAC was autogenerated; false indicates it was set manually
         * First published in XenServer 5.5.
         */
        @JsonProperty("MAC_autogenerated")
        public Boolean MACAutogenerated;

        /**
         * current locking mode of the VIF
         * First published in XenServer 6.1.
         */
        @JsonProperty("locking_mode")
        public Types.VifLockingMode lockingMode;

        /**
         * A list of IPv4 addresses which can be used to filter traffic passing through this VIF
         * First published in XenServer 6.1.
         */
        @JsonProperty("ipv4_allowed")
        public Set<String> ipv4Allowed;

        /**
         * A list of IPv6 addresses which can be used to filter traffic passing through this VIF
         * First published in XenServer 6.1.
         */
        @JsonProperty("ipv6_allowed")
        public Set<String> ipv6Allowed;

        /**
         * Determines whether IPv4 addresses are configured on the VIF
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv4_configuration_mode")
        public Types.VifIpv4ConfigurationMode ipv4ConfigurationMode;

        /**
         * IPv4 addresses in CIDR format
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv4_addresses")
        public Set<String> ipv4Addresses;

        /**
         * IPv4 gateway (the empty string means that no gateway is set)
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv4_gateway")
        public String ipv4Gateway;

        /**
         * Determines whether IPv6 addresses are configured on the VIF
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv6_configuration_mode")
        public Types.VifIpv6ConfigurationMode ipv6ConfigurationMode;

        /**
         * IPv6 addresses in CIDR format
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv6_addresses")
        public Set<String> ipv6Addresses;

        /**
         * IPv6 gateway (the empty string means that no gateway is set)
         * First published in XenServer 7.0.
         */
        @JsonProperty("ipv6_gateway")
        public String ipv6Gateway;

    }

    /**
     * Get a record containing the current state of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return all fields from the object
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public VIF.Record getRecord(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_record";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<VIF.Record>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get a reference to the VIF instance with the specified UUID.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param uuid UUID of object to return
     * @return reference to the object
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public static VIF getByUuid(Connection c, String uuid) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_by_uuid";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, uuid};
        var typeReference = new TypeReference<VIF>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Create a new VIF instance, and return its handle.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param record All constructor arguments
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public static Task createAsync(Connection c, VIF.Record record) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.create";
        String sessionReference = c.getSessionReference();
        var record_map = record.toMap();
        Object[] methodParameters = {sessionReference, record_map};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Create a new VIF instance, and return its handle.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param record All constructor arguments
     * @return reference to the newly created object
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public static VIF create(Connection c, VIF.Record record) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.create";
        String sessionReference = c.getSessionReference();
        var record_map = record.toMap();
        Object[] methodParameters = {sessionReference, record_map};
        var typeReference = new TypeReference<VIF>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Destroy the specified VIF instance.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task destroyAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.destroy";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Destroy the specified VIF instance.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void destroy(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.destroy";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Get the uuid field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getUuid(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_uuid";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the allowed_operations field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<Types.VifOperations> getAllowedOperations(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_allowed_operations";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<Types.VifOperations>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the current_operations field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Map<String, Types.VifOperations> getCurrentOperations(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_current_operations";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Map<String, Types.VifOperations>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the device field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getDevice(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_device";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the network field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Network getNetwork(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_network";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Network>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the VM field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public VM getVM(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_VM";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<VM>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the MAC field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getMAC(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_MAC";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the MTU field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Long getMTU(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_MTU";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Long>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the other_config field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Map<String, String> getOtherConfig(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_other_config";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Map<String, String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the currently_attached field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Boolean getCurrentlyAttached(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_currently_attached";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Boolean>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the status_code field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Long getStatusCode(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_status_code";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Long>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the status_detail field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getStatusDetail(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_status_detail";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the runtime_properties field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Map<String, String> getRuntimeProperties(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_runtime_properties";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Map<String, String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the qos/algorithm_type field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getQosAlgorithmType(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_qos_algorithm_type";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the qos/algorithm_params field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Map<String, String> getQosAlgorithmParams(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_qos_algorithm_params";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Map<String, String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the qos/supported_algorithms field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<String> getQosSupportedAlgorithms(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_qos_supported_algorithms";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the metrics field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     * @deprecated since XenServer 6.1
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    @Deprecated(since = "XenServer 6.1")
    public VIFMetrics getMetrics(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_metrics";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<VIFMetrics>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the MAC_autogenerated field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 5.5.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Boolean getMACAutogenerated(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_MAC_autogenerated";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Boolean>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the locking_mode field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Types.VifLockingMode getLockingMode(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_locking_mode";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Types.VifLockingMode>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv4_allowed field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<String> getIpv4Allowed(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv6_allowed field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<String> getIpv6Allowed(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv4_configuration_mode field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Types.VifIpv4ConfigurationMode getIpv4ConfigurationMode(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv4_configuration_mode";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Types.VifIpv4ConfigurationMode>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv4_addresses field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<String> getIpv4Addresses(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv4_addresses";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv4_gateway field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getIpv4Gateway(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv4_gateway";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv6_configuration_mode field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Types.VifIpv6ConfigurationMode getIpv6ConfigurationMode(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv6_configuration_mode";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Types.VifIpv6ConfigurationMode>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv6_addresses field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Set<String> getIpv6Addresses(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv6_addresses";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Set<String>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Get the ipv6_gateway field of the given VIF.
     * Minimum allowed role: read-only
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @return value of the field
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public String getIpv6Gateway(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_ipv6_gateway";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<String>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Set the other_config field of the given VIF.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param otherConfig New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setOtherConfig(Connection c, Map<String, String> otherConfig) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_other_config";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, otherConfig};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Add the given key-value pair to the other_config field of the given VIF.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to add
     * @param value Value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void addToOtherConfig(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.add_to_other_config";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, key, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Remove the given key and its corresponding value from the other_config field of the given VIF.  If the key is not in that Map, then do nothing.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void removeFromOtherConfig(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.remove_from_other_config";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, key};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Set the qos/algorithm_type field of the given VIF.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param algorithmType New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setQosAlgorithmType(Connection c, String algorithmType) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_qos_algorithm_type";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, algorithmType};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Set the qos/algorithm_params field of the given VIF.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param algorithmParams New value to set
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setQosAlgorithmParams(Connection c, Map<String, String> algorithmParams) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_qos_algorithm_params";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, algorithmParams};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Add the given key-value pair to the qos/algorithm_params field of the given VIF.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to add
     * @param value Value to add
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void addToQosAlgorithmParams(Connection c, String key, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.add_to_qos_algorithm_params";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, key, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Remove the given key and its corresponding value from the qos/algorithm_params field of the given VIF.  If the key is not in that Map, then do nothing.
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @param key Key to remove
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void removeFromQosAlgorithmParams(Connection c, String key) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.remove_from_qos_algorithm_params";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, key};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Hotplug the specified VIF, dynamically attaching it to the running VM
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task plugAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.plug";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Hotplug the specified VIF, dynamically attaching it to the running VM
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void plug(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.plug";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Hot-unplug the specified VIF, dynamically unattaching it from the running VM
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task unplugAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.unplug";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Hot-unplug the specified VIF, dynamically unattaching it from the running VM
     * Minimum allowed role: vm-admin
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void unplug(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.unplug";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Forcibly unplug the specified VIF
     * Minimum allowed role: vm-admin
     * First published in XenServer 6.0.
     *
     * @param c The connection the call is made on
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task unplugForceAsync(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.unplug_force";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Forcibly unplug the specified VIF
     * Minimum allowed role: vm-admin
     * First published in XenServer 6.0.
     *
     * @param c The connection the call is made on
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void unplugForce(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.unplug_force";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Move the specified VIF to the specified network, even while the VM is running
     * Minimum allowed role: vm-admin
     * First published in XenServer 7.1.
     *
     * @param c The connection the call is made on
     * @param network The network to move it to
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task moveAsync(Connection c, Network network) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.move";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, network};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Move the specified VIF to the specified network, even while the VM is running
     * Minimum allowed role: vm-admin
     * First published in XenServer 7.1.
     *
     * @param c The connection the call is made on
     * @param network The network to move it to
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void move(Connection c, Network network) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.move";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, network};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Set the locking mode for this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The new locking mode for the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task setLockingModeAsync(Connection c, Types.VifLockingMode value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.set_locking_mode";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Set the locking mode for this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The new locking mode for the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setLockingMode(Connection c, Types.VifLockingMode value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_locking_mode";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Set the IPv4 addresses to which traffic on this VIF can be restricted
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP addresses which will be associated with the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task setIpv4AllowedAsync(Connection c, Set<String> value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.set_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Set the IPv4 addresses to which traffic on this VIF can be restricted
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP addresses which will be associated with the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setIpv4Allowed(Connection c, Set<String> value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Associates an IPv4 address with this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be associated with the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task addIpv4AllowedAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.add_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Associates an IPv4 address with this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be associated with the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void addIpv4Allowed(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.add_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Removes an IPv4 address from this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be removed from the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task removeIpv4AllowedAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.remove_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Removes an IPv4 address from this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be removed from the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void removeIpv4Allowed(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.remove_ipv4_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Set the IPv6 addresses to which traffic on this VIF can be restricted
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP addresses which will be associated with the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task setIpv6AllowedAsync(Connection c, Set<String> value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.set_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Set the IPv6 addresses to which traffic on this VIF can be restricted
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP addresses which will be associated with the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void setIpv6Allowed(Connection c, Set<String> value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.set_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Associates an IPv6 address with this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be associated with the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task addIpv6AllowedAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.add_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Associates an IPv6 address with this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be associated with the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void addIpv6Allowed(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.add_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Removes an IPv6 address from this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be removed from the VIF
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task removeIpv6AllowedAsync(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.remove_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Removes an IPv6 address from this VIF
     * Minimum allowed role: pool-operator
     * First published in XenServer 6.1.
     *
     * @param c The connection the call is made on
     * @param value The IP address which will be removed from the VIF
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void removeIpv6Allowed(Connection c, String value) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.remove_ipv6_allowed";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, value};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Configure IPv4 settings for this virtual interface
     * Minimum allowed role: vm-operator
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param mode Whether to use static or no IPv4 assignment
     * @param address The IPv4 address in &lt;addr&gt;/&lt;prefix length&gt; format (for static mode only)
     * @param gateway The IPv4 gateway (for static mode only; leave empty to not set a gateway)
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task configureIpv4Async(Connection c, Types.VifIpv4ConfigurationMode mode, String address, String gateway) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.configure_ipv4";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, mode, address, gateway};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Configure IPv4 settings for this virtual interface
     * Minimum allowed role: vm-operator
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param mode Whether to use static or no IPv4 assignment
     * @param address The IPv4 address in &lt;addr&gt;/&lt;prefix length&gt; format (for static mode only)
     * @param gateway The IPv4 gateway (for static mode only; leave empty to not set a gateway)
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void configureIpv4(Connection c, Types.VifIpv4ConfigurationMode mode, String address, String gateway) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.configure_ipv4";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, mode, address, gateway};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Configure IPv6 settings for this virtual interface
     * Minimum allowed role: vm-operator
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param mode Whether to use static or no IPv6 assignment
     * @param address The IPv6 address in &lt;addr&gt;/&lt;prefix length&gt; format (for static mode only)
     * @param gateway The IPv6 gateway (for static mode only; leave empty to not set a gateway)
     * @return Task
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public Task configureIpv6Async(Connection c, Types.VifIpv6ConfigurationMode mode, String address, String gateway) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "Async.VIF.configure_ipv6";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, mode, address, gateway};
        var typeReference = new TypeReference<Task>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Configure IPv6 settings for this virtual interface
     * Minimum allowed role: vm-operator
     * First published in XenServer 7.0.
     *
     * @param c The connection the call is made on
     * @param mode Whether to use static or no IPv6 assignment
     * @param address The IPv6 address in &lt;addr&gt;/&lt;prefix length&gt; format (for static mode only)
     * @param gateway The IPv6 gateway (for static mode only; leave empty to not set a gateway)
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public void configureIpv6(Connection c, Types.VifIpv6ConfigurationMode mode, String address, String gateway) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.configure_ipv6";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference, this.ref, mode, address, gateway};
        c.dispatch(methodCall, methodParameters);
    }

    /**
     * Return a list of all the VIFs known to the system.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return references to all objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public static Set<VIF> getAll(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_all";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference};
        var typeReference = new TypeReference<Set<VIF>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

    /**
     * Return a map of VIF references to VIF records for all VIFs known to the system.
     * Minimum allowed role: read-only
     * First published in XenServer 4.0.
     *
     * @param c The connection the call is made on
     * @return records of all objects
     * @throws BadServerResponse Thrown if the response from the server contains an invalid status.
     * @throws XenAPIException if the call failed.
     * @throws IOException if an error occurs during a send or receive. This includes cases where a payload is invalid JSON.
    */
    public static Map<VIF, VIF.Record> getAllRecords(Connection c) throws
       BadServerResponse,
       XenAPIException,
       IOException {
        String methodCall = "VIF.get_all_records";
        String sessionReference = c.getSessionReference();
        Object[] methodParameters = {sessionReference};
        var typeReference = new TypeReference<Map<VIF, VIF.Record>>(){};
        return c.dispatch(methodCall, methodParameters, typeReference);
    }

}